==> v2.2.0.zip: rse-model/RSPModelLib/include/RSPModel_datatypes.h <==
/**
 * @file		RSPModel_datatypes.h
 * @date		11/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		RSP Model workflow structures definitions
 *
 */

#ifndef RSPMODELLIB_DATATYPES_H_
#define RSPMODELLIB_DATATYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////
//                         Configuration structure
///////////////////////////////////////////////////////////////////////////////

typedef union u_rspmodel_parameters {
	struct {
		unsigned char verbose_mode:1;	//!< Output simple details about ID and other "light" things.
		unsigned char debug_mode:1;		//!< Output all values of faces, indices and vertices and others "heavy" things.
		unsigned char god_mode:1;		//!< Dev only. Output experimental values.

		unsigned char reserved0:5;		//!< For future use.
	};
	unsigned char raw;	//!< Raw options access for bit-masking or memory copy/compare.
} RSPMODEL_PARAMETERS ;


////////////////////////////////////////////////////////////////////////////////
//                         Lib's structure definitions
////////////////////////////////////////////////////////////////////////////////

#ifndef MEMFILE_DEF
typedef char* MEMFILE;
#define MEMFILE_DEF
#endif

#ifndef T_RGBA_DEF
typedef unsigned int T_RGBA;
#define T_RGBA_DEF
#endif

#ifndef T_VECTOR3_DEF
typedef struct vector3 { float x,y,z; } T_VECTOR3;
#define T_VECTOR3_DEF
#endif

#ifndef T_VERTEX_DEF
typedef struct vertex { short x,y,z,w; } T_VERTEX;
#define T_VERTEX_DEF
#endif

#ifndef T_TEXCOORD_DEF
typedef struct tex_coord { short u,v; } T_TEXCOORD;
#define T_TEXCOORD_DEF
#endif

typedef struct face_flags {
	unsigned int fUnknown0:1;
	unsigned int fUnknown1:1;
	unsigned int fHasTexture:1;
	unsigned int fIsQuad:1;
	unsigned int fSeparateColorVertex:1;
	unsigned int fHasColor:1;
	unsigned int fHasExtraBytesBeforeColor:1;
	unsigned int fUnknown7:1;
	unsigned int fUnknown8:1;
	unsigned int fUnknown9:1;
	unsigned int fUnknown10:1;

	unsigned int reserved:21;
} FACE_FLAGS;

typedef struct hob_face {
	union {
		unsigned int flags;
		FACE_FLAGS flags_bits;
	};
	unsigned char b1;
	unsigned char b2;
	unsigned char b3;
	unsigned char bsize;
	unsigned short mt_index;				// Material/texture index, if texture it's correspond to texture_index, otherwise it's mat_'X'
	unsigned short indices[4];
	T_RGBA vertex_colors[4];				//TODO: convert in R:8_G:8_B:8_A:8 format? Caution with BE/LE conversion
	T_TEXCOORD tex_coords[4];
} T_RSPMODEL_FACE;

typedef struct rspmodel_obj_parts {
	unsigned int meshdef1_offset;

	unsigned int face_block_end_offset;
	unsigned int face_block_offset;
	unsigned int vertex_block_offset;

	unsigned int id;
	T_VECTOR3 transform;

	unsigned int face_count;
	T_RSPMODEL_FACE* faces;

	unsigned int vertex_count;
	T_VERTEX* vertices;
} T_RSPMODEL_OBJ_PARTS;

typedef struct rspmodel_object {
	char name[16];
	unsigned int face_group_offset;
	unsigned int object_part_header_offset;
	unsigned int face_group_header_offset;

	unsigned int object_part_count;
	unsigned int face_group_count;

	T_RSPMODEL_OBJ_PARTS* object_parts;
} T_RSPMODEL_OBJECT;

/**
 * @brief	Model-Extractor HOB structure of an HOB file content.
 * @details Used with malloc to create a clean method of bufferized
 *  		model datas before saving it.
 * @todo	Export format to use it directly in other program.
 */
typedef struct rspmodel_hob {
	unsigned int obj_count;
	T_RSPMODEL_OBJECT* objects;
} T_RSPMODEL_HOB;

#ifdef __cplusplus
}
#endif

#endif /* RSPMODELLIB_DATATYPES_H_ */

==> v2.3.0.zip: rse-model/RSPModelLib/include/RSPModel_datatypes.h <==
/**
 * @file		RSPModel_datatypes.h
 * @date		11/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		RSP Model workflow structures definitions
 *
 */

#ifndef RSPMODELLIB_DATATYPES_H_
#define RSPMODELLIB_DATATYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////
//                         Configuration structure
///////////////////////////////////////////////////////////////////////////////

typedef union u_rspmodel_parameters {
	struct {
		unsigned char verbose_mode:1;	//!< Output simple details about ID and other "light" things.
		unsigned char debug_mode:1;		//!< Output all values of faces, indices and vertices and others "heavy" things.
		unsigned char god_mode:1;		//!< Dev only. Output experimental values.

		unsigned char reserved0:5;		//!< For future use.
	};
	unsigned char raw;	//!< Raw options access for bit-masking or memory copy/compare.
} RSPMODEL_PARAMETERS ;


////////////////////////////////////////////////////////////////////////////////
//                         Lib's structure definitions
////////////////////////////////////////////////////////////////////////////////

#ifndef MEMFILE_DEF
typedef char* MEMFILE;
#define MEMFILE_DEF
#endif

typedef unsigned int T_RSPMODEL_RGBA;

typedef struct rspmodel_vector3 { float x,y,z; } T_RSPMODEL_VECTOR3;
typedef struct rspmodel_vertex { short x,y,z,w; } T_RSPMODEL_VERTEX;

typedef struct rspmodel_texcoord { short u,v; } T_RSPMODEL_TEXCOORD;

typedef struct face_flags {
	unsigned int fUnknown0:1;
	unsigned int fUnknown1:1;
	unsigned int fHasTexture:1;
	unsigned int fIsQuad:1;
	unsigned int fSeparateColorVertex:1;
	unsigned int fHasColor:1;
	unsigned int fHasExtraBytesBeforeColor:1;
	unsigned int fUnknown7:1;
	unsigned int fUnknown8:1;
	unsigned int fUnknown9:1;
	unsigned int fUnknown10:1;

	unsigned int reserved:21;
} FACE_FLAGS;

typedef struct hob_face {
	union {
		unsigned int flags;
		FACE_FLAGS flags_bits;
	};
	unsigned char b1;
	unsigned char b2;
	unsigned char b3;
	unsigned char bsize;
	unsigned short mt_index;				// Material/texture index, if texture it's correspond to texture_index, otherwise it's mat_'X'
	unsigned short indices[4];
	T_RSPMODEL_RGBA vertex_colors[4];				//TODO: convert in R:8_G:8_B:8_A:8 format? Caution with BE/LE conversion
	T_RSPMODEL_TEXCOORD tex_coords[4];
} T_RSPMODEL_FACE;

typedef struct rspmodel_obj_parts {
	unsigned int meshdef1_offset;

	unsigned int face_block_end_offset;
	unsigned int face_block_offset;
	unsigned int vertex_block_offset;

	unsigned int id;
	T_RSPMODEL_VECTOR3 transform;

	unsigned int face_count;
	T_RSPMODEL_FACE* faces;

	unsigned int vertex_count;
	T_RSPMODEL_VERTEX* vertices;
} T_RSPMODEL_OBJ_PARTS;

typedef struct rspmodel_object {
	char name[16];
	unsigned int face_group_offset;
	unsigned int object_part_header_offset;
	unsigned int face_group_header_offset;

	unsigned int object_part_count;
	unsigned int face_group_count;

	T_RSPMODEL_OBJ_PARTS* object_parts;
} T_RSPMODEL_OBJECT;

/**
 * @brief	Model-Extractor HOB structure of an HOB file content.
 * @details Used with malloc to create a clean method of bufferized
 *  		model datas before saving it.
 * @todo	Export format to use it directly in other program.
 */
typedef struct rspmodel_hob {
	unsigned int obj_count;
	T_RSPMODEL_OBJECT* objects;
} T_RSPMODEL_HOB;

#ifdef __cplusplus
}
#endif

#endif /* RSPMODELLIB_DATATYPES_H_ */
